#pragma once

// C++ system level
#include <map>
#include <vector>
// #include <cstring>     // memset, strncmp
// #include <cstdlib>     // TODO: is this for memcpy?
// #include <functional>

// ESP32 specific
#include "esp_log.h"

// project specific
#include <Types.hpp>
#include "Service.hpp"

// qthing stuff
// #include <qthing>

// misc
#include <nlohmann/json.hpp>


namespace SiliconTorch {

  namespace ServiceManager {

    extern const str NVSStateNameSpace;


    using ServiceSet = std::set<Service*>;
    using ServiceMap = std::map<str, SiliconTorch::ServiceManager::Service*>;


    class ServiceManager {
      public:

        ServiceManager();

        // Starts registered services and locks registration
        void start();

        void printServiceTable() const;

        nlohmann::json getServiceJSON() const;
        void publishServiceJSON(bool prettyPrint = false) const;

        void setServiceEnabled(const str& name, bool enabled);
        bool getServiceEnabled(const str& name) const;

        std::vector<Service*> getEnabledServicesInStartOrder() const;

        Service* byName(const str& name);

        void registerService(Service* s);

        str deviceTopic(const str& suffix) const;

      private:

        bool running = false;  // also locks service registration
        ServiceMap serviceMap;

        u32 startEnabledServices();

    };


  }
}
